/*
 *  identitymanager.cpp  -  manages the list of email identities
 *  Program:  kalarm
 *  Copyright (C) 2004 by David Jarvie <software@astrojar.org.uk>
 *  This is a modified version of the KDEPIM file of the same name,
 *  Copyright (c) 2002 Marc Mutz <mutz@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qtl.h>
#include <qregexp.h>

#include <kdeversion.h>
#include <kdebug.h>
#include <kconfig.h>

#include "identity.h"
#include "identitymanager.moc"


namespace KPIM {

IdentityManager::IdentityManager(bool, QObject* parent, const char* name) :
#if KDE_VERSION >= 210
	  DCOPObject(),
#endif
	  QObject(parent, name),
	  mOldConfig(false)
{
	// First try to get identity data from the config file "emailidentities"
	mConfig = new KConfig(QString::fromLatin1("emailidentities"), true);
	readConfig();
	if (mIdentities.isEmpty())
	{
		// No emailidentities file, or an empty one due to broken conversion (kconf_update bug
		// in kdelibs <= 3.2.2 => convert it, i.e. read settings from kmailrc
		kdDebug(5950) << "emailidentities is empty -> convert from kmailrc" << endl;
		mOldConfig = true;
		delete mConfig;
		mConfig = new KConfig(QString::fromLatin1("kmailrc"), true);
		readConfig();
	}
#if KDE_VERSION >= 210
	// Receive notification by "KPIM::IdentityManager" of changes to identities
	if (!connectDCOPSignal(0, "KPIM::IdentityManager", "identitiesChanged(QCString,QCString)",
	                       "slotIdentitiesChanged(QCString,QCString)", false))
		kdError(5950) << "IdentityManager: connection to identitiesChanged() failed" << endl;
#endif
}

IdentityManager::~IdentityManager()
{
	delete mConfig;
}

QStringList IdentityManager::identities() const
{
	const_cast<IdentityManager*>(this)->readConfig();     // refresh in case KMail has changed things
	QStringList result;
	for (ConstIterator it = mIdentities.begin();  it != mIdentities.end();  ++it)
		result << (*it).identityName();
	return result;
}

const Identity& IdentityManager::identityForName(const QString& name) const
{
	for (ConstIterator it = begin();  it != end();  ++it)
		if ((*it).identityName() == name)
			return *it;
	return Identity::null;
}

const Identity& IdentityManager::defaultIdentity() const
{
	if (mIdentities.isEmpty())
		return Identity::null;
	for (ConstIterator it = begin();  it != end();  ++it)
		if ((*it).isDefault())
			return *it;
	return *begin();
}

void IdentityManager::slotIdentitiesChanged(QCString, QCString)
{
	mConfig->reparseConfiguration();
	readConfig();
	emit changed();
}

void IdentityManager::readConfig()
{
	mIdentities.clear();
	QStringList identities = mConfig->groupList().grep(QRegExp(mOldConfig ? "^Identity" : "^Identity #\\d+$"));
	if (identities.isEmpty())
		return;   // nothing to be done...

	uint defaultUOID = 0;
	if (!mOldConfig)
	{
		// Find the default identity's UOID
		mConfig->setGroup(QString::fromLatin1("General"));
		defaultUOID = mConfig->readUnsignedNumEntry(QString::fromLatin1("Default Identity"));
	}
	bool haveDefault = false;

	for (QStringList::Iterator group = identities.begin();  group != identities.end();  ++group)
	{
		mConfig->setGroup(*group);
		Iterator last = mIdentities.append(Identity());
		Identity& identity = *last;
		identity.readConfig(mConfig);
		if (identity.isNull())
			mIdentities.remove(last);
		else if (!haveDefault)
		{
			if (mOldConfig  &&  identity.identityName() == QString::fromLatin1("Default")
			||  !mOldConfig  &&  identity.uoid() == defaultUOID)
			{
				haveDefault = true;
				identity.setIsDefault(true);
			}
		}
	}
	if (!haveDefault  &&  !mIdentities.isEmpty())
	{
		kdWarning(5950) << "IdentityManager: There was no default email identity. Marking first one as default." << endl;
		mIdentities.first().setIsDefault(true);
	}
	qHeapSort(mIdentities);
}

} // namespace KPIM
